@@ -21,9 +21,12 @@ email                : markey@web.de
 #include "app.h"
 #include "config.h"
 #include "configdialog.h"
+#include "collectionbrowser.h"
+#include "debug.h"
 #include "effectwidget.h"
 #include "enginebase.h"
 #include "enginecontroller.h"
+#include "equalizersetup.h"
 #include "firstrunwizard.h"
 #include "metabundle.h"
 #include "osd.h"
@@ -32,17 +35,20 @@ email                : markey@web.de
 #include "playlistwindow.h"
 #include "pluginmanager.h"
 #include "scriptmanager.h"
-#include "socketserver.h"
-
+#include "scrobbler.h"
 #include "statusbar.h"
 #include "systray.h"
 #include "tracktooltip.h"        //engineNewMetaData()
 
+#include <iostream>
+
+#include <kconfigdialogmanager.h>
 #include <kcmdlineargs.h>        //initCliArgs()
-#include <kdebug.h>
+#include <kcursor.h>             //amaroK::OverrideCursor
 #include <kedittoolbar.h>        //slotConfigToolbars()
 #include <kglobalaccel.h>        //initGlobalShortcuts()
 #include <kglobalsettings.h>     //applyColorScheme()
+#include <kiconloader.h>         //amarok Icon
 #include <kkeydialog.h>          //slotConfigShortcuts()
 #include <klocale.h>
 #include <kmessagebox.h>         //applySettings(), genericEventHandler()
@@ -61,23 +67,25 @@ email                : markey@web.de
 
 App::App()
         : KApplication()
-        , m_engineToLoad( AmarokConfig::soundSystem() )
         , m_pPlayerWindow( 0 ) //will be created in applySettings()
 {
+    DEBUG_BLOCK
+
     const KCmdLineArgs* const args = KCmdLineArgs::parsedArgs();
-    bool restoreSession = args->count() == 0 || args->isSet( "append" )  || args->isSet( "enqueue" );
+    bool restoreSession = args->count() == 0 || args->isSet( "append" ) || args->isSet( "enqueue" );
 
     QPixmap::setDefaultOptimization( QPixmap::MemoryOptim );
 
     //needs to be created before the wizard
-    m_pDcopHandler    = new amaroK::DcopHandler();
+    new amaroK::DcopPlayerHandler(); // Must be created first
+    new amaroK::DcopPlaylistHandler();
+    new amaroK::DcopCollectionHandler();
 
     // Remember old folder setup, so we can detect changes after the wizard was used
     const QStringList oldCollectionFolders = AmarokConfig::collectionFolders();
 
     if ( amaroK::config()->readBoolEntry( "First Run", true ) || args->isSet( "wizard" ) ) {
-        //stop the splashscreen first, socket server is a temporary on purpose!
-        LoaderServer server( 0 );
+        std::cout << "STARTUP\n" << std::flush; //hide the splashscreen
         firstRunWizard();
         amaroK::config()->writeEntry( "First Run", false );
         amaroK::config()->sync();
@@ -86,15 +94,6 @@ App::App()
     m_pGlobalAccel    = new KGlobalAccel( this );
     m_pPlaylistWindow = new PlaylistWindow();
     m_pTray           = new amaroK::TrayIcon( m_pPlaylistWindow );
-    (void)              new Vis::SocketServer( this );
-
-    #ifdef HAVE_KJSEMBED
-    //FIXME deactivated due to leaking memory
-//     (void)              new ScriptManager::Manager( this );
-    // Export symbols to KJSEmbed
-//     ScriptManager::Manager::instance()->addObject( this );
-//     ScriptManager::Manager::instance()->addObject( m_pDcopHandler );
-    #endif
 
     m_pPlaylistWindow->init(); //creates the playlist, browsers, etc.
     initGlobalShortcuts();
@@ -103,12 +102,24 @@ App::App()
     if ( restoreSession && AmarokConfig::savePlaylist() )
         Playlist::instance()->restoreSession();
 
+    if( args->isSet( "engine" ) ) {
+        // we correct some common errors (case issues, missing -engine off the end)
+        QString engine = args->getOption( "engine" ).lower();
+        if( engine.startsWith( "gstreamer" ) ) engine = "gst-engine";
+        if( !engine.endsWith( "engine" ) ) engine += "-engine";
+
+        AmarokConfig::setSoundSystem( engine );
+    }
+
     //create engine, show PlayerWindow, show TrayIcon etc.
     applySettings( true );
 
-    //initializes Unix domain socket for loader communication, and hides the splash
-    //do here so splash is hidden just after amaroK's windows appear
-    (void) new LoaderServer( this );
+    // Start ScriptManager. Must be created _after_ PlaylistWindow.
+    if ( amaroK::config( "ScriptManager" )->readBoolEntry( "Auto Run" ) )
+        ScriptManager::instance();
+
+    //notify loader application that we have started
+    std::cout << "STARTUP\n" << std::flush;
 
     //after this point only analyzer and temporary pixmaps will be created
     QPixmap::setDefaultOptimization( QPixmap::BestOptim );
@@ -133,39 +144,48 @@ App::App()
 
     // Trigger collection scan if folder setup was changed by wizard
     if ( oldCollectionFolders != AmarokConfig::collectionFolders() )
-        emit sigScanCollection();
+        CollectionDB::instance()->startScan();
+    // If database version is updated, the collection needs to be rescanned.
+    // Works also if the collection is empty for some other reason
+    // (e.g. deleted collection.db)
+    else if ( CollectionDB::instance()->isEmpty() )
+    {
+        CollectionDB::instance()->startScan();
+    }
 
     handleCliArgs();
 }
 
 App::~App()
 {
-    kdDebug() << k_funcinfo << endl;
+    DEBUG_BLOCK
 
     // Hiding the OSD before exit prevents crash
     amaroK::OSD::instance()->hide();
 
     EngineBase* const engine = EngineController::engine();
 
-    if( AmarokConfig::resumePlayback() )
-    {
-        if( engine->state() != Engine::Empty )
-        {
+    if ( AmarokConfig::resumePlayback() ) {
+        if ( engine->state() != Engine::Empty ) {
             AmarokConfig::setResumeTrack( EngineController::instance()->playingURL().prettyURL() );
             AmarokConfig::setResumeTime( engine->position() );
         }
         else AmarokConfig::setResumeTrack( QString::null ); //otherwise it'll play previous resume next time!
     }
 
-    //records final statistics
-    EngineController::instance()->endSession();
+    EngineController::instance()->endSession(); //records final statistics
+    EngineController::instance()->detach( this );
 
-    //do even if trayicon is not shown, it is safe
+    // do even if trayicon is not shown, it is safe
     amaroK::config()->writeEntry( "HiddenOnExit", mainWindow()->isHidden() );
 
     delete m_pPlayerWindow;   //sets some XT keys
     delete m_pPlaylistWindow; //sets some XT keys
-    delete m_pDcopHandler;
+
+    // this must be deleted before the connection to the Xserver is
+    // severed, or we risk a crash when the QApplication is exited,
+    // I asked Trolltech! *smug*
+    delete amaroK::OSD::instance();
 
     AmarokConfig::setVersion( APP_VERSION );
     AmarokConfig::writeConfig();
@@ -182,16 +202,21 @@ void App::handleCliArgs() //static
     if ( args->count() > 0 )
     {
         KURL::List list;
-
-        for ( int i = 0; i < args->count(); i++ )
+        for( int i = 0; i < args->count(); i++ )
             list << args->url( i );
 
-        if ( args->isSet( "queue" ) )
-            Playlist::instance()->queueMedia( list, args->isSet( "play" ) );
-        else if( args->isSet( "append" ) || args->isSet( "enqueue" ) )
-            Playlist::instance()->appendMedia( list, args->isSet( "play" ) );
+        int options;
+        if( args->isSet( "queue" ) )
+           options = Playlist::Queue;
+        else if( args->isSet( "append" ) || args->isSet( "enqueue" ) ) {
+           options = Playlist::Append;
+           if( args->isSet( "play" ) )
+              options |= Playlist::DirectPlay;
+        }
         else
-            Playlist::instance()->replaceMedia( list, true );
+           options = Playlist::Replace | Playlist::DirectPlay;
+
+        Playlist::instance()->insertMedia( list, options );
     }
 
     //we shouldn't let the user specify two of these since it is pointless!
@@ -212,7 +237,7 @@ void App::handleCliArgs() //static
     else if ( args->isSet( "previous" ) )
         EngineController::instance()->previous();
 
-    if (  args->isSet( "toggle-playlist-window" ) )
+    if ( args->isSet( "toggle-playlist-window" ) )
         pApp->m_pPlaylistWindow->showHide();
 
     args->clear();    //free up memory
@@ -250,6 +275,7 @@ void App::initCliArgs( int argc, char *argv[] ) //static
             { "m", 0, 0 },
             { "toggle-playlist-window", I18N_NOOP("Toggle the Playlist-window"), 0 },
             { "wizard", I18N_NOOP( "Run First-run Wizard" ), 0 },
+            { "engine <name>", I18N_NOOP( "Use the <name> engine" ), 0 },
             { 0, 0, 0 }
         };
 
@@ -265,7 +291,6 @@ void App::initGlobalShortcuts()
 {
     EngineController* const ec = EngineController::instance();
 
-
     m_pGlobalAccel->insert( "play", i18n( "Play" ), 0, KKey("WIN+x"), 0,
                             ec, SLOT( play() ), true, true );
     m_pGlobalAccel->insert( "pause", i18n( "Pause" ), 0, KKey("WIN+c"), 0,
@@ -282,12 +307,18 @@ void App::initGlobalShortcuts()
                             ec, SLOT( increaseVolume() ), true, true );
     m_pGlobalAccel->insert( "voldn", i18n( "Decrease Volume" ), 0, KKey("WIN+KP_Subtract"), 0,
                             ec, SLOT( decreaseVolume() ), true, true );
-    m_pGlobalAccel->insert( "playlist_add", i18n( "Add Media" ), 0, KKey("WIN+a"), 0,
+    m_pGlobalAccel->insert( "seekforward", i18n( "Seek Forward" ), 0, KKey("WIN+Shift+KP_Add"), 0,
+                            ec, SLOT( seekForward() ), true, true );
+    m_pGlobalAccel->insert( "seekbackward", i18n( "Seek Backward" ), 0, KKey("WIN+Shift+KP_Subtract"), 0,
+                            ec, SLOT( seekBackward() ), true, true );
+    m_pGlobalAccel->insert( "playlist_add", i18n( "Add Media..." ), 0, KKey("WIN+a"), 0,
                             m_pPlaylistWindow, SLOT( slotAddLocation() ), true, true );
     m_pGlobalAccel->insert( "show", i18n( "Toggle Playlist Window" ), 0, KKey("WIN+p"), 0,
                             m_pPlaylistWindow, SLOT( showHide() ), true, true );
     m_pGlobalAccel->insert( "osd", i18n( "Show OSD" ), 0, KKey("WIN+o"), 0,
-                            amaroK::OSD::instance(), SLOT( forceShowTrack() ), true, true );
+                            amaroK::OSD::instance(), SLOT( forceToggleOSD() ), true, true );
+    m_pGlobalAccel->insert( "mute", i18n( "Mute Volume" ), 0, KKey("WIN+m"), 0,
+                            ec, SLOT( mute() ), true, true );
 
     m_pGlobalAccel->setConfigGroup( "Shortcuts" );
     m_pGlobalAccel->readSettings( kapp->config() );
@@ -296,7 +327,7 @@ void App::initGlobalShortcuts()
     //TODO fix kde accel system so that kactions find appropriate global shortcuts
     //     and there is only one configure shortcuts dialog
 
-    KActionCollection* const ac = actionCollection();
+    KActionCollection* const ac = amaroK::actionCollection();
     KAccelShortcutList list( m_pGlobalAccel );
 
     for( uint i = 0; i < list.count(); ++i )
@@ -327,10 +358,6 @@ class ID3v1StringHandler : public TagLib::ID3v1::StringHandler
 {
     QTextCodec *m_codec;
 
-public:
-    ID3v1StringHandler( int codecIndex )
-        : m_codec( QTextCodec::codecForIndex( codecIndex ) ) {}
-
     virtual TagLib::String parse( const TagLib::ByteVector &data ) const
     {
         return QStringToTString( m_codec->toUnicode( data.data(), data.size() ) );
@@ -341,6 +368,14 @@ public:
         const QCString qcs = m_codec->fromUnicode( TStringToQString( ts ) );
         return TagLib::ByteVector( qcs, qcs.length() );
     }
+
+public:
+    ID3v1StringHandler( int codecIndex )
+            : m_codec( QTextCodec::codecForIndex( codecIndex ) )
+    {
+        debug() << "codec: " << m_codec << endl;
+        debug() << "codec-name: " << m_codec->name() << endl;
+    }
 };
 
 //SLOT
@@ -348,7 +383,7 @@ void App::applySettings( bool firstTime )
 {
     ///Called when the configDialog is closed with OK or Apply
 
-    kdDebug() << "BEGIN " << k_funcinfo << endl;
+    DEBUG_BLOCK
 
     //determine and apply colors first
     applyColorScheme();
@@ -390,28 +425,23 @@ void App::applySettings( bool firstTime )
         //m_pPlaylistWindow->show(); //must be shown //we do below now
 
         //ensure that at least one Menu is plugged into an accessible UI element
-        if( !AmarokConfig::showMenuBar() && !actionCollection()->action( "amarok_menu" )->isPlugged() )
+        if( !AmarokConfig::showMenuBar() && !amaroK::actionCollection()->action( "amarok_menu" )->isPlugged() )
            playlistWindow()->createGUI();
     }
 
 
+    playlistWindow()->applySettings();
+    Scrobbler::instance()->applySettings();
     amaroK::OSD::instance()->applySettings();
-
-    playlistWindow()->setFont(
-        AmarokConfig::useCustomFonts()
-            ? AmarokConfig::playlistWindowFont()
-            : QApplication::font(),
-        AmarokConfig::useCustomFonts()
-            ? AmarokConfig::contextBrowserFont()
-            : QApplication::font() );
-
-
+    CollectionDB::instance()->applySettings();
     amaroK::StatusBar::instance()->setShown( AmarokConfig::showStatusBar() );
-
     m_pTray->setShown( AmarokConfig::showTrayIcon() );
 
-    if ( AmarokConfig::recodeID3v1Tags() )
-        TagLib::ID3v1::Tag::setStringHandler( new ID3v1StringHandler( AmarokConfig::tagEncoding() ) );
+
+    // we check > 0 because textCodecForIndex( 0 ) crashes amaroK for unknown
+    // reasons, also now we assign index 0 to "" in the config combobox
+    if( AmarokConfig::recodeID3v1Tags() && AmarokConfig::recodeEncoding() > 0 )
+        TagLib::ID3v1::Tag::setStringHandler( new ID3v1StringHandler( AmarokConfig::recodeEncoding() ) );
 
 
     //on startup we need to show the window, but only if it wasn't hidden on exit
@@ -428,13 +458,11 @@ void App::applySettings( bool firstTime )
     { //<Engine>
         EngineBase *engine = EngineController::engine();
 
-        if ( firstTime ) AmarokConfig::setSoundSystem( m_engineToLoad );
-
         if( firstTime || AmarokConfig::soundSystem() !=
                          PluginManager::getService( engine )->property( "X-KDE-amaroK-name" ).toString() )
         {
             //will unload engine for us first if necessary
-            engine = EngineController::loadEngine();
+            engine = EngineController::instance()->loadEngine();
         }
 
         AmarokConfig::setHardwareMixer( engine->setHardwareMixer( AmarokConfig::hardwareMixer() ) );
@@ -442,111 +470,105 @@ void App::applySettings( bool firstTime )
         engine->setXfadeLength( AmarokConfig::crossfade() ? AmarokConfig::crossfadeLength() : 0 );
         engine->setVolume( AmarokConfig::masterVolume() );
 
+        engine->setEqualizerEnabled( AmarokConfig::equalizerEnabled() );
+        if ( AmarokConfig::equalizerEnabled() )
+            engine->setEqualizerParameters( AmarokConfig::equalizerPreamp(), AmarokConfig::equalizerGains() );
     } //</Engine>
 
-    /* delete unneeded cover images from cache */
-    QString size = QString::number( AmarokConfig::coverPreviewSize() ) + '@';
-    QDir cacheDir = KGlobal::dirs()->saveLocation( "data", "amarok/" ) + "albumcovers/cache/";
-    QStringList obsoleteCovers = cacheDir.entryList( "*" );
-    for( QStringList::Iterator it = obsoleteCovers.begin(); it != obsoleteCovers.end(); ++it )
-        if ( !( *it ).startsWith( size  ) && !( *it ).startsWith( "50@" ) )
-            QFile( cacheDir.filePath( *it ) ).remove();
+    { //<Collection>
+        CollectionView::instance()->renderView();
+    } //</Collection>
 
-    kdDebug() << "END " << k_funcinfo << endl;
+    {   // delete unneeded cover images from cache
+        const QString size = QString::number( AmarokConfig::coverPreviewSize() ) + '@';
+        const QDir cacheDir = amaroK::saveLocation( "albumcovers/cache/" );
+        const QStringList obsoleteCovers = cacheDir.entryList( "*" );
+        foreach( obsoleteCovers )
+            if ( !(*it).startsWith( size  ) && !(*it).startsWith( "50@" ) )
+                QFile( cacheDir.filePath( *it ) ).remove();
+    }
+
+    //if ( !firstTime )
+        // Bizarrely and ironically calling this causes crashes for
+        // some people! FIXME
+        //AmarokConfig::writeConfig();
 }
 
-
-void App::applyColorScheme()
+void
+App::applyColorScheme()
 {
-    //TODO shorter, better placement etc.
+    DEBUG_BLOCK
+
+    QColorGroup group;
+    using amaroK::ColorScheme::AltBase;
+    int h, s, v;
+    QWidget* const browserBar = (QWidget*)playlistWindow()->child( "BrowserBar" );
 
     if( AmarokConfig::schemeKDE() )
     {
-        QObject* const browserBar = m_pPlaylistWindow->child( "BrowserBar" );
-        QObjectList* const list = browserBar->queryList( "QWidget" );
-        list->prepend( browserBar );
+        AltBase = KGlobalSettings::alternateBackgroundColor();
 
-        kdDebug() << "Discovered " << list->count() << " widgets in PlaylistWindow\n";
-
-        for( QObject *o = list->first(); o; o = list->next() )
-        {
-            //We have to unset the palette due to BrowserWin::setColors() setting
-            //some widgets' palettes, and thus they won't propagate the changes
-
-            static_cast<QWidget*>(o)->unsetPalette();
-
-            //FIXME this is hack for our greyed out text search box thingies (eg FileBrowser)
-            if ( qstrcmp( o->name(), "filter_edit" ) == 0 ) {
-                QEvent e( QEvent::FocusOut );
-                App::sendEvent( o, &e );
-            }
-
-            if( o->inherits( "KListView" ) )
-            {
-                //TODO find out how KListView alternate colors are updated when a
-                //     control center colour change is made
-
-                static_cast<KListView*>(o)->setAlternateBackground( KGlobalSettings::alternateBackgroundColor() );
-            }
-        }
-
-        delete list;
+        playlistWindow()->unsetPalette();
+        browserBar->unsetPalette();
 
         PlayerWidget::determineAmarokColors();
     }
 
     else if( AmarokConfig::schemeAmarok() )
     {
-        QColorGroup group = QApplication::palette().active();
+        group = QApplication::palette().active();
         const QColor bg( amaroK::blue );
-        const QColor bgAlt( 57, 64, 98 );
-
-        /* PLEASE don't do this, it makes lots of widget ugly
-         * instead customise BrowserWin::setColors();
-         * OR figure out wicked colour scheme!
-         */
-        //group.setColor( QColorGroup::Foreground, Qt::white );
+        AltBase.setRgb( 57, 64, 98 );
 
         group.setColor( QColorGroup::Text, Qt::white );
         group.setColor( QColorGroup::Base, bg );
-        group.setColor( QColorGroup::Background, bg.light(120) );
+        group.setColor( QColorGroup::Foreground, 0xd7d7ef );
+        group.setColor( QColorGroup::Background, AltBase );
+
+        group.setColor( QColorGroup::Button, AltBase );
+        group.setColor( QColorGroup::ButtonText, 0xd7d7ef );
+
+//         group.setColor( QColorGroup::Light,    Qt::cyan   /*lighter than Button color*/ );
+//         group.setColor( QColorGroup::Midlight, Qt::blue   /*between Button and Light*/ );
+//         group.setColor( QColorGroup::Dark,     Qt::green  /*darker than Button*/ );
+//         group.setColor( QColorGroup::Mid,      Qt::red    /*between Button and Dark*/ );
+//         group.setColor( QColorGroup::Shadow,   Qt::yellow /*a very dark color. By default, the shadow color is Qt::black*/ );
 
-        group.setColor( QColorGroup::Dark, Qt::white );
         group.setColor( QColorGroup::Highlight, Qt::white );
         group.setColor( QColorGroup::HighlightedText, bg );
         //group.setColor( QColorGroup::BrightText, QColor( 0xff, 0x40, 0x40 ) ); //GlowColor
 
-        int h,s,v;
-        bgAlt.getHsv( &h, &s, &v );
+        AltBase.getHsv( &h, &s, &v );
         group.setColor( QColorGroup::Midlight, QColor( h, s/3, (int)(v * 1.2), QColor::Hsv ) ); //column separator in playlist
 
         //TODO set all colours, even button colours, that way we can change the dark,
         //light, etc. colours and amaroK scheme will look much better
 
-        m_pPlaylistWindow->setColors( QPalette( group, group, group ), bgAlt );
-
         using namespace amaroK::ColorScheme;
         Base       = amaroK::blue;
         Text       = Qt::white;
         Background = 0x002090;
         Foreground = 0x80A0FF;
+
+        //all children() derive their palette from this
+        playlistWindow()->setPalette( QPalette( group, group, group ) );
+        browserBar->unsetPalette();
     }
 
     else if( AmarokConfig::schemeCustom() )
     {
         // we try to be smart: this code figures out contrasting colors for
         // selection and alternate background rows
-        QColorGroup group = QApplication::palette().active();
+        group = QApplication::palette().active();
         const QColor fg( AmarokConfig::playlistWindowFgColor() );
         const QColor bg( AmarokConfig::playlistWindowBgColor() );
-        int h, s, v;
 
         //TODO use the ensureContrast function you devised in BlockAnalyzer
 
         bg.hsv( &h, &s, &v );
         v += (v < 128) ? +50 : -50;
         v &= 255; //ensures 0 <= v < 256
-        QColor bgAlt( h, s, v, QColor::Hsv );
+        AltBase.setHsv( h, s, v );
 
         fg.hsv( &h, &s, &v );
         v += (v < 128) ? +150 : -150;
@@ -559,12 +581,20 @@ void App::applyColorScheme()
         group.setColor( QColorGroup::Highlight, highlight );
         group.setColor( QColorGroup::HighlightedText, Qt::white );
         group.setColor( QColorGroup::Dark, Qt::darkGray );
-        //group.setColor( QColorGroup::BrightText, QColor( 0xff, 0x40, 0x40 ) ); //GlowColor
-
-        m_pPlaylistWindow->setColors( QPalette( group, group, group ), bgAlt );
 
         PlayerWidget::determineAmarokColors();
+
+        // we only colour the middle section since we only
+        // allow the user to choose two colours
+        browserBar->setPalette( QPalette( group, group, group ) );
+        playlistWindow()->unsetPalette();
     }
+
+    // set the KListView alternate colours
+    QObjectList* const list = playlistWindow()->queryList( "KListView" );
+    for( QObject *o = list->first(); o; o = list->next() )
+        static_cast<KListView*>(o)->setAlternateBackground( AltBase );
+    delete list; //heap allocated!
 }
 
 
@@ -594,9 +624,9 @@ bool amaroK::genericEventHandler( QWidget *recipient, QEvent *e )
             //FIXME this isn't a good way to determine if there is a currentTrack, need playlist() function
             const bool b = EngineController::engine()->loaded();
 
-            popup.insertItem( i18n( "&Append to Playlist" ), 101 );
-            popup.insertItem( i18n( "Append && &Play" ), 102 );
-            if( b ) popup.insertItem( i18n( "&Queue After Current Track" ), 103 );
+            popup.insertItem( i18n( "&Append to Playlist" ), Playlist::Append );
+            popup.insertItem( i18n( "Append && &Play" ), Playlist::DirectPlay | Playlist::Append );
+            if( b ) popup.insertItem( i18n( "&Queue After Current Track" ), Playlist::Queue );
             popup.insertSeparator();
             popup.insertItem( i18n( "&Cancel" ), 0 );
 
@@ -604,15 +634,8 @@ bool amaroK::genericEventHandler( QWidget *recipient, QEvent *e )
             KURL::List list;
             KURLDrag::decode( e, list );
 
-            switch( id )
-            {
-            case 101:
-            case 102:
-                Playlist::instance()->appendMedia( list, id == 102 );
-
-            case 103:
-                Playlist::instance()->queueMedia( list );
-            }
+            if ( id > 0 )
+                Playlist::instance()->insertMedia( list, id );
         }
         else return false;
         #undef e
@@ -642,8 +665,7 @@ bool amaroK::genericEventHandler( QWidget *recipient, QEvent *e )
         }
         case Qt::ShiftButton:
         {
-            int seek = EngineController::engine()->position() + ( e->delta() / 120 ) * 10000; // 10 seconds
-            EngineController::engine()->seek( seek < 0 ? 0 : seek );
+            EngineController::instance()->seekRelative( ( e->delta() / 120 ) * 10000 ); // 10 seconds
             break;
         }
         default:
@@ -686,19 +708,19 @@ void App::engineStateChanged( Engine::State state )
 {
     switch( state )
     {
-        case Engine::Empty:
-            QToolTip::add( m_pTray, i18n( "amaroK - Audio Player" ) );
-            break;
+    case Engine::Empty:
+        QToolTip::add( m_pTray, i18n( "amaroK - Audio Player" ) );
+        break;
 
-        default:
-            break;
+    default:
+        ;
     }
 }
 
 
 void App::engineNewMetaData( const MetaBundle &bundle, bool /*trackChanged*/ )
 {
-    amaroK::OSD::instance()->showTrack( bundle );
+    amaroK::OSD::instance()->show( bundle );
 
     TrackToolTip::add( m_pTray, bundle );
 }
@@ -706,10 +728,13 @@ void App::engineNewMetaData( const MetaBundle &bundle, bool /*trackChanged*/ )
 
 void App::engineVolumeChanged( int newVolume )
 {
-    amaroK::OSD::instance()->showOSD( i18n("Volume: %1%").arg( newVolume ), true );
+    //get the amarok icon to show in osd if not playing - else show album cover.
+    QImage osdImage = QImage::QImage();
+    Engine::Playing ? osdImage = QImage::QImage()  :  osdImage = QImage( KIconLoader().iconPath( "amarok", -KIcon::SizeHuge ) );
+    amaroK::OSD::instance()->OSDWidget::show( newVolume ? i18n("Volume: %1%").arg( newVolume ) : i18n("Mute") );
 }
 
-void App::slotConfigEffects( bool show )
+void App::slotConfigEffects( bool show ) //SLOT
 {
     if( show )
     {
@@ -731,11 +756,17 @@ void App::slotConfigEffects( bool show )
 }
 
 
-void App::slotConfigAmarok( int page )
+void App::slotConfigEqualizer() //SLOT
 {
-    kdDebug() << k_funcinfo << endl;
+    EqualizerSetup::instance()->raise();
+}
 
-    KConfigDialog* dialog = KConfigDialog::exists( "settings" );
+
+void App::slotConfigAmarok( const QCString& page )
+{
+    DEBUG_FUNC_INFO
+
+    AmarokConfigDialog* dialog = (AmarokConfigDialog*) KConfigDialog::exists( "settings" );
 
     if( !dialog )
     {
@@ -745,18 +776,22 @@ void App::slotConfigAmarok( int page )
         connect( dialog, SIGNAL(settingsChanged()), SLOT(applySettings()) );
     }
 
+    //FIXME it seems that if the dialog is on a different desktop it gets lost
+    //      what do to? detect and move it?
+
     dialog->show();
+    dialog->raise();
+    dialog->setActiveWindow();
 
     //so that if the engine page is needed to be shown it works
     kapp->processEvents();
 
-    dialog->showPage( page );
-
+    if ( !page.isNull() ) dialog->showPage( page );
 }
 
 void App::slotConfigShortcuts()
 {
-    KKeyDialog::configure( actionCollection(), m_pPlaylistWindow );
+    KKeyDialog::configure( amaroK::actionCollection(), m_pPlaylistWindow );
 }
 
 void App::slotConfigGlobalShortcuts()
@@ -784,6 +819,11 @@ void App::firstRunWizard()
     ///show firstRunWizard
 
     FirstRunWizard wizard;
+    setTopWidget( &wizard );
+    KConfigDialogManager* config = new KConfigDialogManager(&wizard, AmarokConfig::self(), "wizardconfig");
+    config->updateWidgets();
+   // connect(config, SIGNAL(settingsChanged()), SLOT(updateSettings()));
+    wizard.setCaption( makeStdCaption( i18n( "First-Run Wizard" ) ) );
 
     if( wizard.exec() != QDialog::Rejected )
     {
@@ -792,7 +832,9 @@ void App::firstRunWizard()
         case FirstRunWizard::XMMS:
             amaroK::config()->writeEntry( "XMLFile", "amarokui_xmms.rc" );
             AmarokConfig::setShowPlayerWindow( true );
-            AmarokConfig::setShowStatusBar( false );
+            //FIXME the statusbar is now quite essential and also without it
+            // the popup messages break. Fix in 1.2.1
+            AmarokConfig::setShowStatusBar( /*false*/ true );
             break;
 
         case FirstRunWizard::Compact:
@@ -808,7 +850,9 @@ void App::firstRunWizard()
         // If wizard is invoked at runtime, rescan collection if folder setup has changed
         if ( !amaroK::config()->readBoolEntry( "First Run", true ) &&
              oldCollectionFolders != AmarokConfig::collectionFolders() )
-            emit sigScanCollection();
+            CollectionDB::instance()->startScan();
+
+        config->updateSettings();
     }
 }
 
@@ -820,8 +864,8 @@ void App::pruneCoverImages()
 
     const int MAX_DAYS = 90;
 
-    QDir covers( KGlobal::dirs()->saveLocation( "data", kapp->instanceName() + "/albumcovers/" ) );
-    QDir coverCache( KGlobal::dirs()->saveLocation( "data", kapp->instanceName() + "/albumcovers/cache/" ) );
+    QDir covers( amaroK::saveLocation( "albumcovers/large/" ) );
+    QDir coverCache( amaroK::saveLocation( "albumcovers/cache/" ) );
 
     QFileInfoList list( *covers.entryInfoList( QDir::Files ) );
     QFileInfoList listCache( *coverCache.entryInfoList( QDir::Files ) );
@@ -835,25 +879,35 @@ void App::pruneCoverImages()
     uint pruneCount = 0;
     const QDate currentDate = QDate::currentDate();
     for ( uint i = 0; i < list.count(); ++i, ++count )
-        if ( list.at( i )->created().date().daysTo( currentDate ) > MAX_DAYS )
-        {
+        if ( list.at( i )->created().date().daysTo( currentDate ) > MAX_DAYS ) {
             pruneCount++;
             QFile::remove( list.at( i )->absFilePath() );
         }
 
-    kdDebug() << "[App] Pruned " << pruneCount << " of " << count << " amazon cover images.\n";
+    if ( pruneCount )
+        amaroK::StatusBar::instance()->longMessage( i18n(
+                "amaroK has had to delete some of the images that you downloaded from Amazon. "
+                "Amazon's licensing policy require us to remove content within 90 days of its download. "
+                "You can read more about Amazon's policies "
+                "<a href='http://www.amazon.com/webservices'>here</a>. "
+                "Please note that this only affects covers downloaded using the Cover Manager's fetch covers feature and the covers downloaded using the context menu in the Context browser, not manually added covers." ) );
 #endif
 }
 
-
 QWidget *App::mainWindow() const
 {
-    //mainWindow varies, honest!
-    return AmarokConfig::showPlayerWindow() ? (QWidget*)m_pPlayerWindow : (QWidget*)m_pPlaylistWindow;
+   return AmarokConfig::showPlayerWindow() ? (QWidget*)m_pPlayerWindow : (QWidget*)m_pPlaylistWindow;
 }
 
 namespace amaroK
 {
+    /// @see amarok.h
+
+    QWidget *mainWindow()
+    {
+        return pApp->playlistWindow();
+    }
+
     KActionCollection *actionCollection()
     {
         return pApp->playlistWindow()->actionCollection();
@@ -869,11 +923,34 @@ namespace amaroK
 
     namespace ColorScheme
     {
-        QColor Base; //amaroK::blue
-        QColor Text; //Qt::white
-        QColor Background; //brighter blue
-        QColor Foreground; //lighter blue
+        QColor Base;
+        QColor Text;
+        QColor Background;
+        QColor Foreground;
+        QColor AltBase;
+    }
+
+    OverrideCursor::OverrideCursor( Qt::CursorShape cursor )
+    {
+        QApplication::setOverrideCursor( cursor == Qt::WaitCursor ? KCursor::waitCursor() : KCursor::workingCursor() );
+    }
+
+    OverrideCursor::~OverrideCursor()
+    {
+        QApplication::restoreOverrideCursor();
+    }
+
+    QString saveLocation( const QString &directory )
+    {
+        return KGlobal::dirs()->saveLocation( "data", QString("amarok/") + directory, true );
     }
 }
 
+namespace Debug
+{
+    /// @see debug.h
+
+    QCString indent; //declared in debug.h
+}
+
 #include "app.moc"
